@@ -2,7 +2,6 @@
 #define SQLPARSER_SQLPARSER_H
 
 #include "SQLParserResult.h"
-#include "sql/statements.h"
 
 namespace hsql {
 
@@ -18,12 +17,12 @@ class SQLParser_API SQLParser {
   // Run tokenization on the given string and store the tokens in the output vector.
   static bool tokenize(const std::string& sql, std::vector<int16_t>* tokens);
 
-  // Deprecated.
   // Old method to parse SQL strings. Replaced by parse().
+  POCO_DEPRECATED("Use parse()")
   static bool parseSQLString(const char* sql, SQLParserResult* result);
 
-  // Deprecated.
   // Old method to parse SQL strings. Replaced by parse().
+  POCO_DEPRECATED("Use parse()")
   static bool parseSQLString(const std::string& sql, SQLParserResult* result);
 
  private:
@@ -32,4 +31,4 @@ class SQLParser_API SQLParser {
 
 }  // namespace hsql
 
-#endif
\ No newline at end of file
+#endif
