@@ -50,16 +50,17 @@
 #endif
 
 
-#if POCO_OS == POCO_OS_MAC_OS_X || POCO_OS == POCO_OS_FAMILY_BSD
+#if POCO_OS == POCO_OS_MAC_OS_X || POCO_OS == POCO_OS_FREE_BSD
 #include <sys/uio.h>
 #include <sys/types.h>
-using sighandler_t = sig_t;
 #endif
 
-#if POCO_OS == POCO_OS_LINUX && !defined(POCO_EMSCRIPTEN)
+
+#if POCO_OS == POCO_OS_LINUX && defined(POCO_HAVE_SENDFILE) && !defined(POCO_EMSCRIPTEN)
 #include <sys/sendfile.h>
 #endif
 
+
 #if defined(_MSC_VER)
 #pragma warning(disable:4996) // deprecation warnings
 #endif
@@ -73,7 +74,6 @@ using Poco::Timespan;
 
 
 #ifdef WEPOLL_H_
-
 namespace {
 
 	int close(HANDLE h)
@@ -82,7 +82,6 @@ namespace {
 	}
 
 }
-
 #endif // WEPOLL_H_
 
 
@@ -327,21 +326,23 @@ void SocketImpl::shutdownReceive()
 }
 
 
-void SocketImpl::shutdownSend()
+int SocketImpl::shutdownSend()
 {
 	if (_sockfd == POCO_INVALID_SOCKET) throw InvalidSocketException();
 
 	int rc = ::shutdown(_sockfd, 1);
 	if (rc != 0) error();
+	return 0;
 }
 
 
-void SocketImpl::shutdown()
+int SocketImpl::shutdown()
 {
 	if (_sockfd == POCO_INVALID_SOCKET) throw InvalidSocketException();
 
 	int rc = ::shutdown(_sockfd, 2);
 	if (rc != 0) error();
+	return 0;
 }
 
 
@@ -361,8 +362,10 @@ void SocketImpl::checkBrokenTimeout(SelectMode mode)
 
 int SocketImpl::sendBytes(const void* buffer, int length, int flags)
 {
-	checkBrokenTimeout(SELECT_WRITE);
-
+	if (_blocking)
+	{
+		checkBrokenTimeout(SELECT_WRITE);
+	}
 	int rc;
 	do
 	{
@@ -370,15 +373,26 @@ int SocketImpl::sendBytes(const void* buffer, int length, int flags)
 		rc = ::send(_sockfd, reinterpret_cast<const char*>(buffer), length, flags);
 	}
 	while (_blocking && rc < 0 && lastError() == POCO_EINTR);
-	if (rc < 0) error();
+	if (rc < 0)
+	{
+		int err = lastError();
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
+			;
+		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
+			throw TimeoutException(err);
+		else
+			error(err);
+	}
 	return rc;
 }
 
 
 int SocketImpl::sendBytes(const SocketBufVec& buffers, int flags)
 {
-	checkBrokenTimeout(SELECT_WRITE);
-
+	if (_blocking)
+	{
+		checkBrokenTimeout(SELECT_WRITE);
+	}
 	int rc = 0;
 	do
 	{
@@ -395,15 +409,26 @@ int SocketImpl::sendBytes(const SocketBufVec& buffers, int flags)
 #endif
 	}
 	while (_blocking && rc < 0 && lastError() == POCO_EINTR);
-	if (rc < 0) error();
+	if (rc < 0)
+	{
+		int err = lastError();
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
+			;
+		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
+			throw TimeoutException(err);
+		else
+			error(err);
+	}
 	return rc;
 }
 
 
 int SocketImpl::receiveBytes(void* buffer, int length, int flags)
 {
-	checkBrokenTimeout(SELECT_READ);
-
+	if (_blocking)
+	{
+		checkBrokenTimeout(SELECT_READ);
+	}
 	int rc;
 	do
 	{
@@ -414,7 +439,7 @@ int SocketImpl::receiveBytes(void* buffer, int length, int flags)
 	if (rc < 0)
 	{
 		int err = lastError();
-		if (err == POCO_EAGAIN && !_blocking)
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
 			;
 		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
 			throw TimeoutException(err);
@@ -427,8 +452,10 @@ int SocketImpl::receiveBytes(void* buffer, int length, int flags)
 
 int SocketImpl::receiveBytes(SocketBufVec& buffers, int flags)
 {
-	checkBrokenTimeout(SELECT_READ);
-
+	if (_blocking)
+	{
+		checkBrokenTimeout(SELECT_READ);
+	}
 	int rc = 0;
 	do
 	{
@@ -448,7 +475,7 @@ int SocketImpl::receiveBytes(SocketBufVec& buffers, int flags)
 	if (rc < 0)
 	{
 		int err = lastError();
-		if (err == POCO_EAGAIN && !_blocking)
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
 			;
 		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
 			throw TimeoutException(err);
@@ -476,7 +503,7 @@ int SocketImpl::receiveBytes(Poco::Buffer<char>& buffer, int flags, const Poco::
 		if (rc < 0)
 		{
 			int err = lastError();
-			if (err == POCO_EAGAIN && !_blocking)
+			if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
 				;
 			else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
 				throw TimeoutException(err);
@@ -502,7 +529,16 @@ int SocketImpl::sendTo(const void* buffer, int length, const SocketAddress& addr
 #endif
 	}
 	while (_blocking && rc < 0 && lastError() == POCO_EINTR);
-	if (rc < 0) error();
+	if (rc < 0)
+	{
+		int err = lastError();
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
+			;
+		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
+			throw TimeoutException(err);
+		else
+			error(err);
+	}
 	return rc;
 }
 
@@ -534,7 +570,16 @@ int SocketImpl::sendTo(const SocketBufVec& buffers, const SocketAddress& address
 #endif
 	}
 	while (_blocking && rc < 0 && lastError() == POCO_EINTR);
-	if (rc < 0) error();
+	if (rc < 0)
+	{
+		int err = lastError();
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
+			;
+		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
+			throw TimeoutException(err);
+		else
+			error(err);
+	}
 	return rc;
 }
 
@@ -556,7 +601,10 @@ int SocketImpl::receiveFrom(void* buffer, int length, SocketAddress& address, in
 
 int SocketImpl::receiveFrom(void* buffer, int length, struct sockaddr** ppSA, poco_socklen_t** ppSALen, int flags)
 {
-	checkBrokenTimeout(SELECT_READ);
+	if (_blocking)
+	{
+		checkBrokenTimeout(SELECT_READ);
+	}
 	int rc;
 	do
 	{
@@ -567,7 +615,7 @@ int SocketImpl::receiveFrom(void* buffer, int length, struct sockaddr** ppSA, po
 	if (rc < 0)
 	{
 		int err = lastError();
-		if (err == POCO_EAGAIN && !_blocking)
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
 			;
 		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
 			throw TimeoutException(err);
@@ -595,7 +643,10 @@ int SocketImpl::receiveFrom(SocketBufVec& buffers, SocketAddress& address, int f
 
 int SocketImpl::receiveFrom(SocketBufVec& buffers, struct sockaddr** pSA, poco_socklen_t** ppSALen, int flags)
 {
-	checkBrokenTimeout(SELECT_READ);
+	if (_blocking)
+	{
+		checkBrokenTimeout(SELECT_READ);
+	}
 	int rc = 0;
 	do
 	{
@@ -624,7 +675,7 @@ int SocketImpl::receiveFrom(SocketBufVec& buffers, struct sockaddr** pSA, poco_s
 	if (rc < 0)
 	{
 		int err = lastError();
-		if (err == POCO_EAGAIN && !_blocking)
+		if (!_blocking && (err == POCO_EAGAIN || err == POCO_EWOULDBLOCK))
 			;
 		else if (err == POCO_EAGAIN || err == POCO_ETIMEDOUT)
 			throw TimeoutException(err);
@@ -644,6 +695,25 @@ void SocketImpl::sendUrgent(unsigned char data)
 }
 
 
+std::streamsize SocketImpl::sendFile(FileInputStream& fileInputStream, std::streamoff offset, std::streamsize count)
+{
+	if (!getBlocking()) throw NetException("sendFile() not supported for non-blocking sockets");
+
+#ifdef POCO_HAVE_SENDFILE
+	if (secure())
+	{
+		return sendFileBlockwise(fileInputStream, offset, count);
+	}
+	else
+	{
+		return sendFileNative(fileInputStream, offset, count);
+	}
+#else
+	return sendFileBlockwise(fileInputStream, offset, count);
+#endif
+}
+
+
 int SocketImpl::available()
 {
 	int result = 0;
@@ -1373,12 +1443,15 @@ void SocketImpl::error(int code, const std::string& arg)
 	}
 }
 
+
+#ifdef POCO_HAVE_SENDFILE
 #ifdef POCO_OS_FAMILY_WINDOWS
-Poco::Int64 SocketImpl::sendFile(FileInputStream &fileInputStream, Poco::UInt64 offset)
+
+
+std::streamsize SocketImpl::sendFileNative(FileInputStream& fileInputStream, std::streamoff offset, std::streamsize count)
 {
 	FileIOS::NativeHandle fd = fileInputStream.nativeHandle();
-	Poco::UInt64 fileSize = fileInputStream.size();
-	std::streamoff sentSize = fileSize - offset;
+	if (count == 0) count = fileInputStream.size() - offset;
 	LARGE_INTEGER offsetHelper;
 	offsetHelper.QuadPart = offset;
 	OVERLAPPED overlapped;
@@ -1388,66 +1461,126 @@ Poco::Int64 SocketImpl::sendFile(FileInputStream &fileInputStream, Poco::UInt64
 	overlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
 	if (overlapped.hEvent == nullptr)
 	{
-		return -1;
+		int err = GetLastError();
+		error(err);
 	}
-	bool result = TransmitFile(_sockfd, fd, sentSize, 0, &overlapped, nullptr, 0);
+	bool result = TransmitFile(_sockfd, fd, count, 0, &overlapped, nullptr, 0);
 	if (!result)
 	{
 		int err = WSAGetLastError();
-		if ((err != ERROR_IO_PENDING) && (WSAGetLastError() != WSA_IO_PENDING)) {
+		if ((err != ERROR_IO_PENDING) && (WSAGetLastError() != WSA_IO_PENDING)) 
+		{
 			CloseHandle(overlapped.hEvent);
-			error(err, Error::getMessage(err));
+			error(err);
 		}
 		WaitForSingleObject(overlapped.hEvent, INFINITE);
 	}
 	CloseHandle(overlapped.hEvent);
-	return sentSize;
-}
-#else
-Poco::Int64 _sendfile(poco_socket_t sd, FileIOS::NativeHandle fd, Poco::UInt64 offset,std::streamoff sentSize)
-{
-	Poco::Int64 sent = 0;
-#ifdef __USE_LARGEFILE64
-	sent = sendfile64(sd, fd, (off64_t *)&offset, sentSize);
-#else
-#if POCO_OS == POCO_OS_LINUX && !defined(POCO_EMSCRIPTEN)
-	sent = sendfile(sd, fd, (off_t *)&offset, sentSize);
-#elif POCO_OS == POCO_OS_MAC_OS_X
-	int result = sendfile(fd, sd, offset, &sentSize, nullptr, 0);
-	if (result < 0)
-	{
-		sent = -1;
-	} 
-	else 
-	{
-		sent = sentSize;
-	}
-#else
-	throw Poco::NotImplementedException("sendfile not implemented for this platform");
-#endif
-#endif
-	if (errno == EAGAIN || errno == EWOULDBLOCK) 
-	{
-		sent = 0;
-	}
-	return sent;
+	return count;
 }
 
-Poco::Int64 SocketImpl::sendFile(FileInputStream &fileInputStream, Poco::UInt64 offset)
+
+#else
+
+
+namespace
+{
+	std::streamoff sendFileUnix(poco_socket_t sd, FileIOS::NativeHandle fd, std::streamoff offset, std::streamsize count)
+	{
+		std::streamoff sent = 0;
+		#ifdef __USE_LARGEFILE64
+			off_t noffset = offset;
+			sent = sendfile64(sd, fd, &noffset, count);
+		#else
+			#if POCO_OS == POCO_OS_LINUX && !defined(POCO_EMSCRIPTEN)
+				off_t noffset = offset;
+				sent = sendfile(sd, fd, &noffset, count);
+			#elif POCO_OS == POCO_OS_MAC_OS_X
+				off_t len = count;
+				int result = sendfile(fd, sd, offset, &len, NULL, 0);
+				if (result < 0)
+				{
+					sent = -1;
+				} 
+				else 
+				{
+					sent = len;
+				}
+			#elif POCO_OS == POCO_OS_FREE_BSD
+				off_t sbytes;
+				int result = sendfile(fd, sd, offset, count, NULL, &sbytes, 0);
+				if (result < 0)
+				{
+					sent = -1;
+				} 
+				else 
+				{
+					sent = sbytes;
+				}
+			#else
+				throw Poco::NotImplementedException("native sendfile not implemented for this platform");
+			#endif
+		#endif
+		return sent;
+	}	
+}
+
+
+std::streamsize SocketImpl::sendFileNative(FileInputStream& fileInputStream, std::streamoff offset, std::streamsize count)
 {
 	FileIOS::NativeHandle fd = fileInputStream.nativeHandle();
-	Poco::UInt64 fileSize = fileInputStream.size();
-	std::streamoff sentSize = fileSize - offset;
-	Poco::Int64 sent = 0;
-	sighandler_t sigPrev = signal(SIGPIPE, SIG_IGN);
-	while (sent == 0)
+	if (count == 0) count = fileInputStream.size() - offset;
+	std::streamsize sent = 0;
+	while (count > 0)
 	{
-		errno = 0;
-		sent = _sendfile(_sockfd, fd, offset, sentSize);
+		std::streamoff rc = sendFileUnix(_sockfd, fd, offset, count);
+		if (rc >= 0)
+		{
+			sent += rc;
+			offset += rc;
+			count -= rc;
+		}
+		else
+		{
+			error(errno);
+		}
 	}
-	signal(SIGPIPE, sigPrev != SIG_ERR ? sigPrev : SIG_DFL);
 	return sent;
 }
+
+
 #endif // POCO_OS_FAMILY_WINDOWS
+#endif // POCO_HAVE_SENDFILE
+
+
+std::streamsize SocketImpl::sendFileBlockwise(FileInputStream& fileInputStream, std::streamoff offset, std::streamsize count)
+{
+	fileInputStream.seekg(offset, std::ios_base::beg);
+	Poco::Buffer<char> buffer(8192);
+	std::size_t bufferSize = buffer.size();
+	if (count > 0 && bufferSize > count) bufferSize = count;
+
+	std::streamsize len = 0;
+	fileInputStream.read(buffer.begin(), bufferSize);
+	std::streamsize n = fileInputStream.gcount();
+	while (n > 0 && (count == 0 || len < count))
+	{
+		len += n;
+		sendBytes(buffer.begin(), n);
+		if (count > 0 && len < count)
+		{
+			const std::size_t remaining = count - len;
+			if (bufferSize > remaining) bufferSize = remaining;
+		}
+		if (fileInputStream)
+		{
+			fileInputStream.read(buffer.begin(), bufferSize);
+			n = fileInputStream.gcount();
+		}
+		else n = 0;
+	}
+	return len;
+}
+
 
 } } // namespace Poco::Net
